//+------------------------------------------------------------------+
//|                                                     TDI_auto.mq5 |
//|                                       Copyright 2023, MetaQuotes |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, MetaQuotes"
#property link      "https://www.mql5.com"
#property version   "1.00"

#include <Trade/Trade.mqh>


CTrade trade;
datetime    glTimeBarOpen;
int         MAHandle, TDIHandle;

bool flagBuy = false;
bool flagSell = false;
input bool useOptionalOne = true; //Volatility Band Check
input bool useOptionalTwo = true; //MBL Check

//+------------------------------------------------------------------+
//| Event Handlers                                                   |
//+------------------------------------------------------------------+
int OnInit()
{
   glTimeBarOpen = D'1971.01.01 00:00';

   MAHandle = MA_Init(250, 0, MODE_EMA, PRICE_CLOSE);
   TDIHandle = TDI_Init("TDI.ex5");

   if(MAHandle == -1 || TDIHandle == -1)
      {
         return(INIT_FAILED);
      }

   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   Print("Expert removed");
}
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
{
//---

//--------------------//
   //  NEW BAR CONTROL   //
   //--------------------//

   bool newBar = false;

   //Check for New Bar
   if(glTimeBarOpen != iTime(Symbol(), PERIOD_CURRENT, 0))
      {
         newBar = true;
         glTimeBarOpen = iTime(Symbol(), PERIOD_CURRENT, 0);
      }

   if(newBar == true)
      {
         //--------------------//
         // PRICE & INDICATORS //
         //--------------------//

         //Price
         double close1 = Close(1);

         //Normalization of close price to tick size
         double tickSize = SymbolInfoDouble(_Symbol, SYMBOL_TRADE_TICK_SIZE);
         close1 = round(close1 / tickSize) * tickSize;

         //Buy positions open trades at Ask but close them at Bid
         //Sell positions open trades at Bid but close them at Ask

         double askPrice = SymbolInfoDouble(_Symbol, SYMBOL_ASK);
         double bidPrice = SymbolInfoDouble(_Symbol, SYMBOL_BID);

         //Price must be normalized either to digits or ticksize
         askPrice = round(askPrice / tickSize) * tickSize;
         bidPrice = round(bidPrice / tickSize) * tickSize;

         //Moving Average
         double ma1 = ma(MAHandle, 1);

         //Trend identification
         string TrendSignal = MA_EntrySignal(close1, ma1);
         Print("Trend is: ", TrendSignal);

         //Crosses
         double Rsi, Rsi1, Rsi2, Vbh, Vbh1, Vbh2, Vbl, Vbl1, Vbl2, Mbl, Tsl, Tsl1, Tsl2;  //variables to hold TDI Buffer values for each candle

         Rsi = RSI(0);  //the function is called and candle index passed as param to get current candle value on TDI chart
         Rsi1 = RSI(1);
         Rsi2 = RSI(2);

         Vbh = VBH(1);
         Vbh1 = VBH(1);
         Vbh2 = VBH(2);

         Tsl = TSL(0);
         Tsl1 = TSL(1);
         Tsl2 = TSL(2);

         Vbl = VBL(0);
         Vbl1 = VBL(1);
         Vbl2 = VBL(2);

         Mbl = MBL(0);
         
         //Cases for Buy crosses
         
         if(!flagBuy){ 
           if(Rsi <= 32 && Rsi < Tsl && ((useOptionalOne && Rsi < Vbl) || !useOptionalOne) && ((useOptionalTwo && Mbl <= 50) || !useOptionalTwo))
           flagBuy = true;
         }
           Print("Buy Flag is :",flagBuy);
         
         if(flagBuy && Rsi < Tsl && Rsi > Tsl){
           if(trade.Buy(0.2,_Symbol,askPrice,0,0,"TDI_auto")){
            flagBuy = false;
           }
         }
         
         //Cases for Sell Crosses
          
          if(!flagSell){ 
           if(Rsi >= 68 && Rsi > Tsl && ((useOptionalOne && Rsi > Vbh) || !useOptionalOne) && ((useOptionalTwo && Mbl >= 50) || !useOptionalTwo))
           flagSell = true;
         }
           Print("Sell Flag is :",flagSell);
         
         if(flagSell && Rsi > Tsl && Rsi < Tsl){
           if(trade.Sell(0.2,_Symbol,bidPrice,0,0,"TDI_auto")){
            flagSell = false;
           }
         } 
  
         
/*
         //rsi below 32, vbl, tsl and mbl <= 50

         if(Mbl <= 50 && Rsi2 <= 32 && Vbl2 <= 32 && Rsi2 < Tsl2 && Rsi > Tsl)
            {
               Comment("Strong Long Signal");
               trade.Buy(0.2, _Symbol, askPrice, 0, 0, "Buy by TDI_auto");
               Print("Case One");
            }
         //rsi below 32, vbl and tsl
         if(Mbl > 50 && Rsi2 <= 32 && Rsi > Tsl)
            {
               Comment("Medium long signal ");
               trade.Buy(0.2, _Symbol, askPrice, 0, 0, "Buy by TDI_auto");
               Print("Case Two");

            }
         //rsi below 32 and below tsl
         if(Rsi2 <= 32 && Rsi > Tsl)
            {
               Comment("Weak Long Signal");
               trade.Buy(0.2, _Symbol, askPrice, 0, 0, "Buy by TDI_auto");
               Print("Case three");
            }



         //Cases for Sell crosses

         // tsl rsi above 68, vbh, tsl and mbl >= 50
         if(Mbl >= 50 && Rsi2 >= 68 && Vbl2 >= 68 && Rsi2 > Tsl2 && Rsi < Tsl)
            {
               Comment("Strong Sell Signal");

               trade.Sell(0.2, _Symbol, bidPrice, 0, 0, "Sell by TDI_auto");
               Print("Case One");
            }
         //rsi above 68, vbh and tsl
         if(Mbl < 50 && Rsi2 >= 68 && Rsi < Tsl)
            {
               Comment("Medium Sell signal ");

               trade.Sell(0.2, _Symbol, bidPrice, 0, 0, "Sell by TDI_auto");
               Print("Case Two");
            }
         //rsi above 68 and above
         if(Rsi2 >= 68 && Rsi < Tsl)
            {
               Comment("Weak Sell Signal");

               trade.Sell(0.2, _Symbol, bidPrice, 0, 0, "Sell by TDI_auto");
               Print("Case Three");
            }

*/


      }
}

//+------------------------------------------------------------------+



//+------------------------------------------------------------------+
//| EA FUNCTIONS                                                     |
//+------------------------------------------------------------------+

//+--------+// Price Functions //+--------+//
//+------------------------------------------------------------------+
// Close and Open creates an object array of MqlRates structure
//it sets our array as a series array (so current bar is position 0, previous bar is 1...)
//it copies the bar price information of bars position 0,1 and 2 to our array "bar"
//it returns the close price of the bar object

double Close(int pShift)
{
   MqlRates bar[];                              
   ArraySetAsSeries(bar, true);                 
   CopyRates(_Symbol, PERIOD_CURRENT, 0, 3, bar); 

   return bar[pShift].close;                    
}

//+------------------------------------------------------------------+
double Open(int pShift)
{
   MqlRates bar[];                              
   ArraySetAsSeries(bar, true);                 
   CopyRates(_Symbol, PERIOD_CURRENT, 0, 3, bar); 

   return bar[pShift].open;                     
}


//+--------+// Moving Average Functions //+--------+//

//+------------------------------------------------------------------+
int MA_Init(int pMAPeriod, int pMAShift, ENUM_MA_METHOD pMAMethod, ENUM_APPLIED_PRICE pMAPrice)
{
   //In case of error when initializing the MA, GetLastError() will get the error code and store it in _lastError.
   //ResetLastError will change _lastError variable to 0
   ResetLastError();

   MAHandle = iMA(Symbol(), PERIOD_CURRENT, pMAPeriod, pMAShift, pMAMethod, pMAPrice);

   if(MAHandle == INVALID_HANDLE)
      {
         return -1;
         Print("There was an error creating the MA indicator handle: ", GetLastError());
      }

   Print("MA Indicator handle initialized successfully");

   return MAHandle;
}
//+------------------------------------------------------------------+

//--- create and fill an array with Moving average  --//
//--- buffer values normalize the Value to our symbol's digits and return it---//

double ma(int pMAHandle, int pShift)
{
   ResetLastError();

   
   double ma[];
   ArraySetAsSeries(ma, true);

   bool fillResult = CopyBuffer(pMAHandle, 0, 0, 3, ma);
   if(fillResult == false)
      {
         Print("FILL_ERROR: ", GetLastError());
      }

   double maValue = ma[pShift];

   
   maValue = NormalizeDouble(maValue, _Digits);

   return maValue;
}

//+------------------------------------------------------------------+
//--- Ma_entrysignal determines the trend using one moving average
//--- Muttiple ma can be added and function modifies as required


string MA_EntrySignal(double pPrice1, double pMA1)
{
   string str = "";
   string indicatorValues;

   if(pPrice1 > pMA1)
      {
         str = "LONG";
      }
   else if(pPrice1 < pMA1)
      {
         str = "SHORT";
      }
   else
      {
         str = "NO_TRADE";
      }

   StringConcatenate(indicatorValues, "MA 1: ", DoubleToString(pMA1, _Digits), " | ", "Close 1: ", DoubleToString(pPrice1, _Digits), " | ");

   Print("Indicator Values: ", indicatorValues);

   return str;
}

//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//--- TDI Indicator Functions ---//

int TDI_Init(string pIndicatorPath)
{
   ResetLastError();

   TDIHandle = iCustom(_Symbol, PERIOD_CURRENT, pIndicatorPath);
   if(TDIHandle == INVALID_HANDLE)
      {
         return -1;
         Print("There was an error creating the TDI indicator Handle: ", GetLastError());
      }
   Print("TDI Handle initialized successfully");
   return TDIHandle;
}
//+------------------------------------------------------------------+

//--- The following functions  create and fill an array with TDI --//
//--- buffer values normalize the Value to our symbol's digits and return it---//

double MBL(int pShift)
{
   ResetLastError();

   double MBL[];
   ArraySetAsSeries(MBL, true);

   bool fillResult = CopyBuffer(TDIHandle, 1, 0, 3, MBL);
   if(fillResult == false)
      {
         Print("FILL_ERROR: ", GetLastError());
      }
   double mblValue = MBL[pShift];

   mblValue = NormalizeDouble(mblValue, _Digits);

   return mblValue;
}
//+------------------------------------------------------------------+
double VBL(int pShift)
{
   ResetLastError();

   double VBL[];
   ArraySetAsSeries(VBL, true);

   bool fillResult = CopyBuffer(TDIHandle, 2, 0, 3, VBL);
   if(fillResult == false)
      {
         Print("FILL_ERROR: ", GetLastError());
      }
   double vblValue = VBL[pShift];

   vblValue = NormalizeDouble(vblValue, _Digits);

   return vblValue;
}

//+------------------------------------------------------------------+
double VBH(int pShift)
{
   ResetLastError();

   double VBH[];
   ArraySetAsSeries(VBH, true);

   bool fillResult = CopyBuffer(TDIHandle, 0, 0, 3, VBH);
   if(fillResult == false)
      {
         Print("FILL_ERROR: ", GetLastError());
      }
   double vbhValue = VBH[pShift];

   vbhValue = NormalizeDouble(vbhValue, _Digits);

   return vbhValue;
}

//+------------------------------------------------------------------+
double RSI(int pShift)
{
   ResetLastError();

   double RSI[];
   ArraySetAsSeries(RSI, true);

   bool fillResult = CopyBuffer(TDIHandle, 4, 0, 3, RSI);
   if(fillResult == false)
      {
         Print("FILL_ERROR: ", GetLastError());
      }
   double rsiValue = RSI[pShift];

   rsiValue = NormalizeDouble(rsiValue, _Digits);

   return rsiValue;
}


//+------------------------------------------------------------------+
double TSL(int pShift)
{
   ResetLastError();

   double TSL[];
   ArraySetAsSeries(TSL, true);

   bool fillResult = CopyBuffer(TDIHandle, 3, 0, 3, TSL);
   if(fillResult == false)
      {
         Print("FILL_ERROR: ", GetLastError());
      }
   double tslValue = TSL[pShift];

   tslValue = NormalizeDouble(tslValue, _Digits);

   return tslValue;
}
//+------------------------------------------------------------------+
